
.installed-overlay-box {
	font-size: 80%;
	background-color: @theme_selected_bg_color;
	border-radius: 4px;
	color: @theme_selected_fg_color;
	text-shadow: 0 1px 0 rgba(0,0,0,0.5);
}

.installed-overlay-label { }

.index-title-alignment-software {
	font-weight: bold;
	font-size: 14px;
}


GtkNotebook.main-notebook-software > GtkScrolledWindow {
	border-bottom-width: 0px;
	border-left-width: 0px;
	border-right-width: 0px;
}

.needs-attention {
        background-image: none;
        background-color: shade(@theme_selected_bg_color, 1.1);
        border-radius: 1px;
}

.screenshot-image, .screenshot-image-thumb {
        background-image: none;
        background-color: shade(@theme_bg_color, 0.9);
}
  .screenshot-image { border-radius: 5px; }
  .screenshot-image-thumb { border-radius: 3px; }

.view.tile {
        border-radius: 0;
        border-width: 3px;
        border-image: url("shadow.png") 3 3 3 3 stretch;
        background-image: none;
        background-color: @list_box_bg;
}

.view.tile:hover {
        background-color: @content_view_bg;
}

.application-details-title {
	font-weight: bold;
	font-size: 14px;
}

.application-details-summary {
}

.application-details-description {
}

.error-label {
	text-shadow: none;
}

.folder-label {
	font-size: 9px;
	background-color: #999999;
	border-radius: 4px;
	padding: 4px 8px;
	color: white;
}

.header-label {
	font-size: 11px;
	padding: 6px;
	background-image: none;
	background-color: #babdb6;
	border-color: #000000;
}

.image-list {
	background-color: transparent;
}

GtkBox.star {
	background-color: transparent;
	background-image: none;
}

.button.star {
	-GtkWidget-focus-padding: 0;
	background-color: transparent;
	background-image: none;
	border-image: none;
	border-radius: 0;
	border-width: 0px;
	padding: 0;
}

.star {
	color: #888a85;
}

.star-user {
	color: #edd400;
}

.counter-label {
    color: @theme_selected_fg_color;
    background-color: mix(@theme_selected_bg_color, @theme_selected_fg_color, 0.3);
    font-size: smaller;
    border-radius: 4px;
    padding: 0px 4px;
}
